/*
 * BluSunrize
 * Copyright (c) 2017
 *
 * This code is licensed under "Blu's License of Common Sense"
 * Details can be found in the license file in the root folder of this project
 */
package blusunrize.trauma.common.effects;

import blusunrize.trauma.api.condition.CapabilityTrauma;
import blusunrize.trauma.api.condition.EnumLimb;
import blusunrize.trauma.api.condition.LimbCondition;
import blusunrize.trauma.api.condition.TraumaStatus;
import blusunrize.trauma.api.effects.IEffectTicking;
import blusunrize.trauma.common.Trauma;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemFood;
import net.minecraft.item.ItemPotion;
import net.minecraft.item.ItemStack;
import net.minecraft.potion.PotionEffect;
import net.minecraft.potion.PotionUtils;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.entity.living.LivingEntityUseItemEvent;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import org.apache.commons.lang3.tuple.Pair;

import java.util.Collection;
import java.util.HashMap;
import java.util.UUID;

/**
 * Regularly ticking exhaustion
 *
 * @author BluSunrize
 * @since 23.09.2017
 */
public class EffectReducedSaturation implements IEffectTicking
{
	public EffectReducedSaturation()
	{
		MinecraftForge.EVENT_BUS.register(this);
	}

	@Override
	public String getIndentifier()
	{
		return Trauma.MODID+":ReducedSaturation";
	}

	@Override
	public String getDescription(EntityPlayer player, LimbCondition limbCondition)
	{
		return "desc.trauma.effect.reducedsaturation";
	}

	@Override
	public void tick(EntityPlayer player, LimbCondition limbCondition)
	{
		if(!player.field_70170_p.field_72995_K)
		{
			if(foodMap.containsKey(player.func_110124_au()))
			{
				Pair<Integer, Float> entry = foodMap.remove(player.func_110124_au());
				float mod = limbCondition.getState().ordinal()*.25f;
				player.func_71024_bL().func_75122_a(Math.round(-entry.getLeft()*mod), entry.getRight()*mod);
			}
			if(potionMap.containsKey(player.func_110124_au()))
			{
				Collection<PotionEffect> entry = potionMap.remove(player.func_110124_au());
				float mod = 1-(limbCondition.getState().ordinal()*.25f);
				for(PotionEffect effect : entry)
					if(!effect.func_188419_a().func_76403_b()&&(!effect.func_188419_a().func_76398_f()||effect.func_188419_a().func_188408_i()))
					{
						PotionEffect active = player.func_70660_b(effect.func_188419_a());
						if(active!=null&&effect.func_76459_b()-active.func_76459_b() <= 1)//Initiated just now
						{
							int dur = 60;//Math.round(active.getDuration()*mod);
							int amp = Math.round(active.func_76458_c()*mod);
							PotionEffect replacement = new PotionEffect(active.func_188419_a(), dur, amp, active.func_82720_e(), active.func_188418_e());
							replacement.setCurativeItems(active.getCurativeItems());
							player.func_193076_bZ().put(replacement.func_188419_a(), replacement);
						}
					}
			}
		}
	}

	private static HashMap<UUID, Pair<Integer, Float>> foodMap = new HashMap<>();
	private static HashMap<UUID, Collection<PotionEffect>> potionMap = new HashMap<>();

	@SubscribeEvent(priority = EventPriority.LOWEST)
	public void onUseItemTick(LivingEntityUseItemEvent.Tick event)
	{
		if(!event.isCanceled()&&event.getDuration()==1&&event.getEntityLiving() instanceof EntityPlayer&&!event.getEntity().field_70170_p.field_72995_K)//Last tick, uncancelled
		{
			EntityPlayer player = (EntityPlayer)event.getEntityLiving();
			TraumaStatus status = player.getCapability(CapabilityTrauma.TRAUMA_CAPABILITY, null);
			if(status.getLimbCondition(EnumLimb.ABDOMEN).hasEffect(getIndentifier()))
			{
				ItemStack stack = event.getItem();
				if(stack.func_77973_b() instanceof ItemFood)
					foodMap.put(event.getEntityLiving().func_110124_au(), Pair.of(((ItemFood)stack.func_77973_b()).func_150905_g(stack), ((ItemFood)stack.func_77973_b()).func_150906_h(stack)));
				else if(stack.func_77973_b() instanceof ItemPotion)
					potionMap.put(event.getEntityLiving().func_110124_au(), PotionUtils.func_185189_a(stack));
			}
		}
	}
}
